@@ -33,18 +33,18 @@ import logging
 import os
 import pathlib
 import typing as t
-from warnings import warn
 
 import httpx
 
 import analytix
-from analytix import errors, oauth
+from analytix import errors, oauth, ux
 from analytix.queries import Query
 from analytix.reports import Report
 from analytix.secrets import Secrets
 from analytix.tokens import Tokens
+from analytix.webserver import RequestHandler, Server
 
-log = logging.getLogger(__name__)
+_log = logging.getLogger(__name__)
 
 
 class Analytics:
@@ -60,10 +60,18 @@ class Analytics:
             :obj:`httpx.Client` constructor.
     """
 
-    __slots__ = ("secrets", "_session", "_tokens", "_token_path", "_checked_for_update")
+    __slots__ = (
+        "secrets",
+        "_legacy_auth",
+        "_session",
+        "_tokens",
+        "_token_path",
+        "_checked_for_update",
+    )
 
     def __init__(self, secrets: Secrets, **kwargs: t.Any) -> None:
         self.secrets = secrets
+        self._legacy_auth = False
         self._session = httpx.Client(**kwargs)
         self._tokens: Tokens | None = None
         self._token_path = pathlib.Path()
@@ -92,15 +100,51 @@ class Analytics:
 
     @property
     def authorised(self) -> bool:
-        """Whether this client is authorised."""
+        """Whether this client is authorised. This property is
+        read-only."""
 
         return self._tokens is not None
 
+    @property
+    def legacy_auth(self) -> bool:
+        """Whether to use manual copy/paste authorisation.
+
+        .. warning::
+            Manual copy/paste authorisation is deprecated by Google, and
+            stopped being made available from 1 April. While analytix
+            still provides the ability to use it for now, it is strongly
+            recommended you only do so if you **absolutely** have to
+            (for example, if your firewall does not allow loopback
+            communication).
+
+        .. warning::
+            Manual copy/paste authorisation is not always available. If
+            it is not, analytix will lock up.
+
+        .. versionadded:: 3.4.0
+        """
+
+        return self._legacy_auth
+
+    @legacy_auth.setter
+    def legacy_auth(self, value: bool) -> None:
+        if not isinstance(value, bool):
+            raise NotImplementedError
+
+        if value:
+            ux._warn(
+                "Manual copy/paste authorisation was deprecated by Google; while "
+                "analytix still allows you to use it, there is no guarantee it will "
+                "work"
+            )
+
+        self._legacy_auth = value
+
     def close_session(self) -> None:
         """Close the currently open session."""
 
         self._session.close()
-        log.info("Session closed")
+        _log.info("Session closed")
 
     def check_for_updates(self) -> str | None:
         """Checks for newer versions of analytix.
@@ -111,17 +155,17 @@ class Analytics:
             ascertained.
         """
 
-        log.debug("Checking for updates...")
+        _log.debug("Checking for updates...")
 
         r = self._session.get(analytix.UPDATE_CHECK_URL)
         if r.is_error:
             # If we can't get the info, just ignore it.
-            log.debug("Failed to get version information")
+            _log.debug("Failed to get version information")
             return None
 
         latest = r.json()["info"]["version"]
         if analytix.__version__ != latest:
-            log.warning(
+            _log.warning(
                 f"You do not have the latest stable version of analytix (v{latest})"
             )
 
@@ -134,10 +178,36 @@ class Analytics:
 
         return Tokens.from_file(path)
 
-    def _retrieve_tokens(self) -> Tokens:
-        url, state = oauth.auth_url_and_state(self.secrets)
-        code = input(f"{url}\nEnter code > ")
-        data, headers = oauth.access_data_and_headers(code, self.secrets)
+    def _mcp(self, url: str) -> str:
+        return input(
+            f"You need to authorise analytix; to do so, visit this URL: {url}\n"
+            "Enter code > "
+        )
+
+    def _ws(self, url: str, redirect_uri: str, port: int) -> str:
+        print(f"You need to authorise analytix; to do so, visit this URL: {url}")
+        ws = Server((redirect_uri[7:], port), RequestHandler)
+
+        try:
+            ws.handle_request()
+        except KeyboardInterrupt as exc:
+            raise exc
+        finally:
+            ws.server_close()
+
+        return ws.code
+
+    def _retrieve_tokens(self, redirect_uris: list[str], port: int) -> Tokens:
+        if self.legacy_auth:
+            rd_addr = redirect_uris[0]
+        else:
+            ru = redirect_uris[-1]
+            rd_addr = f"{ru}:{port}"
+
+        url, _ = oauth.auth_url_and_state(self.secrets, rd_addr)
+        code = self._mcp(url) if self.legacy_auth else self._ws(url, ru, port)
+
+        data, headers = oauth.access_data_and_headers(code, self.secrets, rd_addr)
 
         r = self._session.post(self.secrets.token_uri, data=data, headers=headers)
         if r.is_error:
@@ -157,18 +227,28 @@ class Analytics:
         if not self._tokens:
             return False
 
-        log.debug("Checking if token needs to be refreshed...")
+        _log.debug("Checking if token needs to be refreshed...")
         r = self._session.get(analytix.OAUTH_CHECK_URL + self._tokens.access_token)
         return r.is_error
 
-    def refresh_access_token(self) -> None:
-        """Refresh the access token."""
+    def refresh_access_token(self, *, port: int = 8080) -> None:
+        """Refresh the access token.
+
+        Keyword args:
+            port:
+                The port to use for the authorisation webserver when
+                using loopback IP address authorisation. Defaults to
+                8080. This is ignored if analytix is configured to use
+                manual copy/paste authorisation.
+
+                .. versionadded:: 3.4.0
+        """
 
         if not self._tokens:
-            log.warning("There are no tokens to refresh")
+            _log.warning("There are no tokens to refresh")
             return
 
-        log.info("Refreshing access token...")
+        _log.info("Refreshing access token...")
         data, headers = oauth.refresh_data_and_headers(
             self._tokens.refresh_token, self.secrets
         )
@@ -177,17 +257,20 @@ class Analytics:
         if not r.is_error:
             self._tokens.update(r.json())
         else:
-            log.info("Your refresh token has expired; you will need to reauthorise")
-            self._tokens = self._retrieve_tokens()
+            _log.info("Your refresh token has expired; you will need to reauthorise")
+            self._tokens = self._retrieve_tokens(self.secrets.redirect_uris, port)
 
         self._tokens.write(self._token_path)
 
     def authorise(  # nosec B107
-        self, token_path: pathlib.Path | str = ".", *, force: bool = False
+        self,
+        token_path: pathlib.Path | str = ".",
+        *,
+        force: bool = False,
+        port: int = 8080,
     ) -> Tokens:
         """Authorise the client. This is called automatically when
-        needed if not manually called, though if you want to authorise
-        with non-default options, you will need to call this manually.
+        needed if not manually called.
 
         Args:
             token_path:
@@ -201,17 +284,18 @@ class Analytics:
             force:
                 Whether to forcibly authorise the client. Defaults to
                 ``False``.
+            port:
+                The port to use for the authorisation webserver when
+                using loopback IP address authorisation. Defaults to
+                8080. This is ignored if analytix is configured to use
+                manual copy/paste authorisation.
+
+                .. versionadded:: 3.4.0
 
         Returns:
             The tokens the client is authorised with.
         """
 
-        warning = (
-            "Code-based authorisation is deprecated due to a breaking change in the "
-            "Google APIs -- analytix is patching your secrets file where necessary for "
-            "now, but the authorisation method will change in the next version"
-        )
-
         if not isinstance(token_path, pathlib.Path):
             token_path = pathlib.Path(token_path)
 
@@ -221,20 +305,15 @@ class Analytics:
         self._token_path = token_path
 
         if not force:
-            log.info("Attempting to load tokens...")
+            _log.info("Attempting to load tokens...")
             self._tokens = self._try_load_tokens(token_path)
 
         if not self._tokens:
-            log.info("Unable to load tokens; you will need to authorise")
-            self._tokens = self._retrieve_tokens()
-            # Temporary warning regarding authorisation methods.
-            if log.hasHandlers() and log.getEffectiveLevel() <= 30:
-                log.warning(warning)
-            else:
-                warn(warning)
+            _log.info("Unable to load tokens; you will need to authorise")
+            self._tokens = self._retrieve_tokens(self.secrets.redirect_uris, port)
             self._tokens.write(token_path)
 
-        log.info("Authorisation complete!")
+        _log.info("Authorisation complete!")
         return self._tokens
 
     def retrieve(
@@ -255,6 +334,7 @@ class Analytics:
         skip_update_check: bool = False,
         skip_refresh_check: bool = False,
         token_path: pathlib.Path | str = ".",
+        port: int = 8080,
     ) -> Report:
         """Retrieves a report from the YouTube Analytics API.
 
@@ -326,6 +406,13 @@ class Analytics:
                 "tokens.json".
 
                 .. versionadded:: 3.3.0
+            port:
+                The port to use for the authorisation webserver when
+                using loopback IP address authorisation. Defaults to
+                8080. This is ignored if analytix is configured to use
+                manual copy/paste authorisation.
+
+                .. versionadded:: 3.4.0
 
         Returns:
             An instance for working with retrieved data.
@@ -350,21 +437,21 @@ class Analytics:
         if not skip_validation:
             query.validate()
         else:
-            log.warning(
+            _log.warning(
                 "Skipping validation -- invalid requests will count toward your quota"
             )
 
         if not self.authorised or force_authorisation:
-            self.authorise(token_path=token_path, force=force_authorisation)
+            self.authorise(token_path=token_path, force=force_authorisation, port=port)
 
         if (not skip_refresh_check) and self.needs_refresh():
-            self.refresh_access_token()
+            self.refresh_access_token(port=port)
 
         assert self._tokens is not None
         headers = {"Authorization": f"Bearer {self._tokens.access_token}"}
         resp = self._session.get(query.url, headers=headers)
         data = resp.json()
-        log.debug(f"Data retrieved: {data}")
+        _log.debug(f"Data retrieved: {data}")
 
         if next(iter(data)) == "error":
             error = data["error"]
@@ -375,5 +462,5 @@ class Analytics:
 
         assert query.rtype is not None
         report = Report(data, query.rtype)
-        log.info(f"Created report of shape {report.shape}!")
+        _log.info(f"Created report of shape {report.shape}!")
         return report
