@@ -38,7 +38,7 @@ import aiofiles
 
 from analytix.types import SecretT
 
-log = logging.getLogger(__name__)
+_log = logging.getLogger(__name__)
 
 
 @dataclass(frozen=True)
@@ -98,16 +98,12 @@ class Secrets:
         if not isinstance(path, pathlib.Path):
             path = pathlib.Path(path)
 
-        log.debug(f"Loading secrets from {path.resolve()}...")
+        _log.debug(f"Loading secrets from {path.resolve()}...")
 
         with open(path) as f:
             data = json.load(f)["installed"]
 
-        # Temporary patch to account for the breaking change in the API.
-        if "urn:ietf:wg:oauth:2.0:oob" not in data["redirect_uris"]:
-            data["redirect_uris"].insert(0, "urn:ietf:wg:oauth:2.0:oob")
-
-        log.info("Secrets loaded!")
+        _log.info("Secrets loaded!")
         return cls(**data)
 
     @classmethod
@@ -126,16 +122,12 @@ class Secrets:
         if not isinstance(path, pathlib.Path):
             path = pathlib.Path(path)
 
-        log.debug(f"Loading secrets from {path.resolve()}...")
+        _log.debug(f"Loading secrets from {path.resolve()}...")
 
         async with aiofiles.open(path) as f:
             data = json.loads(await f.read())["installed"]
 
-        # Temporary patch to account for the breaking change in the API.
-        if "urn:ietf:wg:oauth:2.0:oob" not in data["redirect_uris"]:
-            data["redirect_uris"].insert(0, "urn:ietf:wg:oauth:2.0:oob")
-
-        log.info("Secrets loaded!")
+        _log.info("Secrets loaded!")
         return cls(**data)
 
     def to_dict(self) -> dict[str, SecretT]:
